package edu.princeton.swing;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * ClipboardTargetManager is kept informed of all 
 */
public class ClipboardTargetManager
{
    private static final Toolkit TOOLKIT = Toolkit.getDefaultToolkit();

    /**
     * The bytes for the cut icon.
     */
    public static final byte CUT_ICON_BYTES[] =
    {
        (byte)0x47, (byte)0x49, (byte)0x46, (byte)0x38, (byte)0x39, (byte)0x61,
        (byte)0x18, (byte)0x00, (byte)0x18, (byte)0x00, (byte)0xB3, (byte)0xFF,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0xFF, (byte)0xFF,
        (byte)0xFF, (byte)0xC0, (byte)0xC0, (byte)0xC0, (byte)0x6B, (byte)0x6B,
        (byte)0xC7, (byte)0x78, (byte)0x78, (byte)0xD4, (byte)0xF2, (byte)0xF2,
        (byte)0xF2, (byte)0xD9, (byte)0xD9, (byte)0xD9, (byte)0x97, (byte)0x97,
        (byte)0x97, (byte)0x8A, (byte)0x8A, (byte)0x8A, (byte)0x86, (byte)0x86,
        (byte)0x86, (byte)0x79, (byte)0x79, (byte)0x79, (byte)0x24, (byte)0x24,
        (byte)0x24, (byte)0x0D, (byte)0x0D, (byte)0x0D, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x21, (byte)0xF9, (byte)0x04, (byte)0x01, (byte)0x00,
        (byte)0x00, (byte)0x02, (byte)0x00, (byte)0x2C, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x18, (byte)0x00, (byte)0x18, (byte)0x00,
        (byte)0x00, (byte)0x04, (byte)0xA1, (byte)0x50, (byte)0xC8, (byte)0x49,
        (byte)0xAB, (byte)0xBD, (byte)0x0C, (byte)0x84, (byte)0x3B, (byte)0x01,
        (byte)0x2B, (byte)0x9C, (byte)0xA6, (byte)0x85, (byte)0xC1, (byte)0x88,
        (byte)0x15, (byte)0x89, (byte)0xB2, (byte)0x55, (byte)0x4A, (byte)0x52,
        (byte)0x7C, (byte)0x5C, (byte)0xFB, (byte)0x82, (byte)0xD5, (byte)0xE7,
        (byte)0x72, (byte)0x92, (byte)0x36, (byte)0x53, (byte)0x5F, (byte)0xB9,
        (byte)0x72, (byte)0xA9, (byte)0x80, (byte)0x4C, (byte)0x12, (byte)0x54,
        (byte)0xE1, (byte)0x26, (byte)0x3D, (byte)0x93, (byte)0x0E, (byte)0x76,
        (byte)0x14, (byte)0xC8, (byte)0x4A, (byte)0x09, (byte)0xDD, (byte)0xAD,
        (byte)0x99, (byte)0x0B, (byte)0xA0, (byte)0x74, (byte)0x3E, (byte)0x6A,
        (byte)0xC7, (byte)0x57, (byte)0xC8, (byte)0x6A, (byte)0x25, (byte)0x19,
        (byte)0x9F, (byte)0x87, (byte)0xF6, (byte)0xC5, (byte)0x8A, (byte)0x7E,
        (byte)0xD4, (byte)0x71, (byte)0xF8, (byte)0xA5, (byte)0x6B, (byte)0x46,
        (byte)0x19, (byte)0x8B, (byte)0x8C, (byte)0x1C, (byte)0x91, (byte)0x39,
        (byte)0x90, (byte)0x2D, (byte)0x8C, (byte)0x01, (byte)0x41, (byte)0xAF,
        (byte)0x06, (byte)0xEC, (byte)0xF7, (byte)0x4C, (byte)0x15, (byte)0x09,
        (byte)0x7C, (byte)0x79, (byte)0x04, (byte)0x0B, (byte)0x01, (byte)0x08,
        (byte)0x80, (byte)0x7E, (byte)0x03, (byte)0x76, (byte)0x16, (byte)0x7E,
        (byte)0x4B, (byte)0x03, (byte)0x1F, (byte)0x79, (byte)0x49, (byte)0x04,
        (byte)0x26, (byte)0x3C, (byte)0x91, (byte)0x20, (byte)0x8F, (byte)0x25,
        (byte)0x95, (byte)0x1B, (byte)0x93, (byte)0x77, (byte)0x12, (byte)0x0A,
        (byte)0x8A, (byte)0x7B, (byte)0x2E, (byte)0x07, (byte)0x84, (byte)0x7A,
        (byte)0x08, (byte)0x68, (byte)0x12, (byte)0x9B, (byte)0x7A, (byte)0x98,
        (byte)0x06, (byte)0x2F, (byte)0xAC, (byte)0x9C, (byte)0x9F, (byte)0xAA,
        (byte)0x0A, (byte)0x19, (byte)0x2E, (byte)0x06, (byte)0x12, (byte)0x05,
        (byte)0x07, (byte)0x1E, (byte)0xA8, (byte)0xB2, (byte)0x13, (byte)0x56,
        (byte)0x56, (byte)0xB7, (byte)0x13, (byte)0x5D, (byte)0xC3, (byte)0x5F,
        (byte)0xC5, (byte)0x11, (byte)0x00, (byte)0x3B, (byte)0xB3, (byte)0xAF,
        (byte)0xCC, (byte)0xB2, (byte)0xAE, (byte)0xCB, (byte)0xB4, (byte)0xB0,
        (byte)0xCC, (byte)0xC2, (byte)0xBE, (byte)0xDC, (byte)0xB7, (byte)0xB3,
        (byte)0xCF, (byte)0xB5, (byte)0xB1, (byte)0xCD, (byte)0xCB, (byte)0xC7,
        (byte)0xE3, (byte)0xCD, (byte)0xCA, (byte)0xE1, (byte)0xCC, (byte)0xC9,
        (byte)0xE0, (byte)0xD5, (byte)0xD2, (byte)0xE8, (byte)0xD4, (byte)0xD1,
        (byte)0xE7, (byte)0xD3, (byte)0xD0, (byte)0xE6, (byte)0xE9, (byte)0xE6,
        (byte)0xFC, (byte)0x9C, (byte)0x98, (byte)0xB8, (byte)0xA2, (byte)0x9E,
        (byte)0xBE, (byte)0xAE, (byte)0xAB, (byte)0xC8, (byte)0xAD, (byte)0xAA,
        (byte)0xC7, (byte)0xAB, (byte)0xA8, (byte)0xC5, (byte)0xB2, (byte)0xAF,
        (byte)0xCC, (byte)0xAD, (byte)0xAA, (byte)0xC6, (byte)0xAB, (byte)0xA8,
        (byte)0xC4, (byte)0xB7, (byte)0xB4, (byte)0xCF, (byte)0xBA, (byte)0xB7,
        (byte)0xD1, (byte)0xB9, (byte)0xB6, (byte)0xD0, (byte)0xC1, (byte)0xBE,
        (byte)0xD8, (byte)0xBE, (byte)0xBB, (byte)0xD4, (byte)0xBC, (byte)0xB9,
        (byte)0xD2, (byte)0xC4, (byte)0xC1, (byte)0xDA, (byte)0xC3, (byte)0xC0,
        (byte)0xD9, (byte)0xC1, (byte)0xBE, (byte)0xD7, (byte)0xDA, (byte)0xD7,
        (byte)0xF1, (byte)0xE1, (byte)0xDE, (byte)0xF6, (byte)0xE2, (byte)0xE0,
        (byte)0xF3, (byte)0xDE, (byte)0xDC, (byte)0xEF, (byte)0xEF, (byte)0xED,
        (byte)0xFF, (byte)0xE9, (byte)0xE7, (byte)0xF8, (byte)0xBE, (byte)0xBC,
        (byte)0xD5, (byte)0xC8, (byte)0xC6, (byte)0xDE, (byte)0xCC, (byte)0xCA,
        (byte)0xE1, (byte)0xCB, (byte)0xC9, (byte)0xE0, (byte)0xCA, (byte)0xC8,
        (byte)0xDF, (byte)0xD3, (byte)0xD1, (byte)0xE7, (byte)0xD0, (byte)0xCE,
        (byte)0xE4, (byte)0xE1, (byte)0xDF, (byte)0xF6, (byte)0xD6, (byte)0xD4,
        (byte)0xE9, (byte)0xD0, (byte)0xCE, (byte)0xE3, (byte)0xE8, (byte)0xE6,
        (byte)0xFC, (byte)0xDD, (byte)0xDB, (byte)0xEF, (byte)0xDC, (byte)0xDA,
        (byte)0xEE, (byte)0xDB, (byte)0xD9, (byte)0xED, (byte)0xF6, (byte)0xF5,
        (byte)0xFF, (byte)0xD2, (byte)0xD0, (byte)0xEA, (byte)0xE4, (byte)0xE3,
        (byte)0xF5, (byte)0xE3, (byte)0xE2, (byte)0xF5, (byte)0xE3, (byte)0xE3,
        (byte)0xF5, (byte)0xD8, (byte)0xD8, (byte)0xDA, (byte)0xD7, (byte)0xD7,
        (byte)0xD9, (byte)0xDA, (byte)0xDA, (byte)0xDA, (byte)0xD9, (byte)0xD9,
        (byte)0xD9, (byte)0xB6, (byte)0xB6, (byte)0xB6, (byte)0x0D, (byte)0x0D,
        (byte)0x0D, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x21, (byte)0xF9, (byte)0x04, (byte)0x01, (byte)0x00,
        (byte)0x00, (byte)0x01, (byte)0x00, (byte)0x2C, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x18, (byte)0x00, (byte)0x18, (byte)0x00,
        (byte)0x00, (byte)0x08, (byte)0xFF, (byte)0x00, (byte)0x03, (byte)0x08,
        (byte)0x1C, (byte)0x48, (byte)0xB0, (byte)0xA0, (byte)0xC1, (byte)0x83,
        (byte)0x89, (byte)0x12, (byte)0x2A, (byte)0x5C, (byte)0xC8, (byte)0x10,
        (byte)0x00, (byte)0x42, (byte)0x44, (byte)0x00, (byte)0x22, (byte)0x4A,
        (byte)0x9C, (byte)0x28, (byte)0x11, (byte)0x51, (byte)0x22, (byte)0x87,
        (byte)0x05, (byte)0x2F, (byte)0x32, (byte)0xDC, (byte)0x98, (byte)0xF0,
        (byte)0xC4, (byte)0x45, (byte)0x83, (byte)0x1A, (byte)0xDF, (byte)0x1C,
        (byte)0xE9, (byte)0x81, (byte)0xE4, (byte)0x4D, (byte)0x16, (byte)0x36,
        (byte)0x45, (byte)0x00, (byte)0x4D, (byte)0x81, (byte)0x92, (byte)0xC8,
        (byte)0x23, (byte)0x46, (byte)0x82, (byte)0x1A, (byte)0x8F, (byte)0xB4,
        (byte)0x09, (byte)0xD4, (byte)0xC6, (byte)0xCF, (byte)0x95, (byte)0x2A,
        (byte)0x44, (byte)0x82, (byte)0x3C, (byte)0x09, (byte)0xD3, (byte)0xF2,
        (byte)0x63, (byte)0x46, (byte)0x00, (byte)0x89, (byte)0xFE, (byte)0x08,
        (byte)0x82, (byte)0x33, (byte)0xC8, (byte)0x8D, (byte)0x95, (byte)0x39,
        (byte)0x44, (byte)0x80, (byte)0x48, (byte)0xF1, (byte)0xD2, (byte)0xF3,
        (byte)0xE5, (byte)0x40, (byte)0x8D, (byte)0x48, (byte)0xDA, (byte)0x08,
        (byte)0xAA, (byte)0x69, (byte)0x07, (byte)0xA7, (byte)0x4E, (byte)0x26,
        (byte)0x4D, (byte)0x41, (byte)0x02, (byte)0x7D, (byte)0xD3, (byte)0x87,
        (byte)0x8F, (byte)0x9F, (byte)0x3C, (byte)0x77, (byte)0xE4, (byte)0x94,
        (byte)0x19, (byte)0xE3, (byte)0x04, (byte)0xAB, (byte)0x4B, (byte)0xAD,
        (byte)0x89, (byte)0xF6, (byte)0x5C, (byte)0xB1, (byte)0x82, (byte)0x45,
        (byte)0x4F, (byte)0x9D, (byte)0x21, (byte)0x66, (byte)0xA2, (byte)0x70,
        (byte)0x59, (byte)0x92, (byte)0xF5, (byte)0x67, (byte)0x22, (byte)0x3C,
        (byte)0x74, (byte)0xA8, (byte)0xD0, (byte)0x31, (byte)0x32, (byte)0x24,
        (byte)0x8E, (byte)0x90, (byte)0x2F, (byte)0x5E, (byte)0x92, (byte)0xD4,
        (byte)0x85, (byte)0x09, (byte)0x74, (byte)0x0D, (byte)0x11, (byte)0x34,
        (byte)0x69, (byte)0xD4, (byte)0xFC, (byte)0x20, (byte)0xF3, (byte)0xA4,
        (byte)0xCB, (byte)0x12, (byte)0x1F, (byte)0x83, (byte)0x9F, (byte)0x02,
        (byte)0x05, (byte)0x74, (byte)0x06, (byte)0x48, (byte)0x90, (byte)0x31,
        (byte)0x62, (byte)0x9A, (byte)0x80, (byte)0x51, (byte)0xB2, (byte)0x45,
        (byte)0x4B, (byte)0x64, (byte)0x81, (byte)0x1A, (byte)0x39, (byte)0x32,
        (byte)0x3C, (byte)0x9B, (byte)0x11, (byte)0xC5, (byte)0x89, (byte)0xD3,
        (byte)0xA8, (byte)0x53, (byte)0xA3, (byte)0x46, (byte)0xE1, (byte)0x13,
        (byte)0xB4, (byte)0xE8, (byte)0xD7, (byte)0x0A, (byte)0x25, (byte)0x53,
        (byte)0x9C, (byte)0x4D, (byte)0xDB, (byte)0xE7, (byte)0x45, (byte)0x43,
        (byte)0x02, (byte)0x24, (byte)0xE4, (byte)0x90, (byte)0xD1, (byte)0x82,
        (byte)0x86, (byte)0x8D, (byte)0x1A, (byte)0x1F, (byte)0x06, (byte)0x28,
        (byte)0x30, (byte)0xD1, (byte)0x61, (byte)0xC3, (byte)0x81, (byte)0x0B,
        (byte)0x05, (byte)0x6C, (byte)0x03, (byte)0x30, (byte)0x14, (byte)0x41,
        (byte)0xC2, (byte)0x8E, (byte)0x07, (byte)0x2E, (byte)0x68, (byte)0xC0,
        (byte)0x00, (byte)0xF1, (byte)0x81, (byte)0x81, (byte)0x8A, (byte)0x0A,
        (byte)0x14, (byte)0x26, (byte)0xA4, (byte)0x08, (byte)0x31, (byte)0x42,
        (byte)0xB9, (byte)0x80, (byte)0x43, (byte)0x38, (byte)0x20, (byte)0xDC,
        (byte)0x33, (byte)0x98, (byte)0x61, (byte)0xC3, (byte)0x41, (byte)0x83,
        (byte)0x0F, (byte)0x1E, (byte)0x12, (byte)0x70, (byte)0xD0, (byte)0x80,
        (byte)0x81, (byte)0x84, (byte)0x88, (byte)0x15, (byte)0xDE, (byte)0x09,
        (byte)0x15, (byte)0xE2, (byte)0xD1, (byte)0xC2, (byte)0x85, (byte)0x8E,
        (byte)0x18, (byte)0x2F, (byte)0x16, (byte)0x10, (byte)0x28, (byte)0x81,
        (byte)0x40, (byte)0x83, (byte)0x81, (byte)0x0C, (byte)0x2C, (byte)0x58,
        (byte)0x60, (byte)0x1B, (byte)0x6C, (byte)0xB0, (byte)0xBD, (byte)0x44,
        (byte)0xDB, (byte)0x81, (byte)0xB3, (byte)0x1D, (byte)0xA4, (byte)0xE0,
        (byte)0x82, (byte)0x01, (byte)0x04, (byte)0x04, (byte)0x00, (byte)0x21,
        (byte)0xFE, (byte)0x4F, (byte)0x43, (byte)0x6F, (byte)0x70, (byte)0x79,
        (byte)0x72, (byte)0x69, (byte)0x67, (byte)0x68, (byte)0x74, (byte)0x20,
        (byte)0x32, (byte)0x30, (byte)0x30, (byte)0x30, (byte)0x20, (byte)0x62,
        (byte)0x79, (byte)0x20, (byte)0x53, (byte)0x75, (byte)0x6E, (byte)0x20,
        (byte)0x4D, (byte)0x69, (byte)0x63, (byte)0x72, (byte)0x6F, (byte)0x73,
        (byte)0x79, (byte)0x73, (byte)0x74, (byte)0x65, (byte)0x6D, (byte)0x73,
        (byte)0x2C, (byte)0x20, (byte)0x49, (byte)0x6E, (byte)0x63, (byte)0x2E,
        (byte)0x20, (byte)0x41, (byte)0x6C, (byte)0x6C, (byte)0x20, (byte)0x52,
        (byte)0x69, (byte)0x67, (byte)0x68, (byte)0x74, (byte)0x73, (byte)0x20,
        (byte)0x52, (byte)0x65, (byte)0x73, (byte)0x65, (byte)0x72, (byte)0x76,
        (byte)0x65, (byte)0x64, (byte)0x2E, (byte)0x0D, (byte)0x0A, (byte)0x4A,
        (byte)0x4C, (byte)0x46, (byte)0x20, (byte)0x47, (byte)0x52, (byte)0x20,
        (byte)0x56, (byte)0x65, (byte)0x72, (byte)0x20, (byte)0x31, (byte)0x2E,
        (byte)0x30, (byte)0x0D, (byte)0x0A, (byte)0x00, (byte)0x3B
    };

    /**
     * The cut icon.
     */
    public static final Icon CUT_ICON = new ImageIcon(
        TOOLKIT.createImage(CUT_ICON_BYTES),
        "Cut"
        );

    /**
     * The bytes for the copy icon.
     */
    public static final byte COPY_ICON_BYTES[] =
    {
        (byte)0x47, (byte)0x49, (byte)0x46, (byte)0x38, (byte)0x39, (byte)0x61,
        (byte)0x18, (byte)0x00, (byte)0x18, (byte)0x00, (byte)0xE6, (byte)0xFF,
        (byte)0x00, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xE7, (byte)0xE7,
        (byte)0xFF, (byte)0xE7, (byte)0xE6, (byte)0xFF, (byte)0xE6, (byte)0xE7,
        (byte)0xFF, (byte)0xE6, (byte)0xE6, (byte)0xFF, (byte)0xE5, (byte)0xE5,
        (byte)0xFF, (byte)0xE5, (byte)0xE5, (byte)0xFE, (byte)0xE4, (byte)0xE4,
        (byte)0xFD, (byte)0xE3, (byte)0xE3, (byte)0xFD, (byte)0xE2, (byte)0xE2,
        (byte)0xFC, (byte)0xE1, (byte)0xE1, (byte)0xFB, (byte)0xE0, (byte)0xE0,
        (byte)0xF9, (byte)0xDF, (byte)0xDF, (byte)0xF9, (byte)0xDE, (byte)0xDE,
        (byte)0xF8, (byte)0xDD, (byte)0xDD, (byte)0xF6, (byte)0xDC, (byte)0xDC,
        (byte)0xF5, (byte)0xDA, (byte)0xDA, (byte)0xF4, (byte)0xD9, (byte)0xDA,
        (byte)0xF3, (byte)0xD9, (byte)0xD9, (byte)0xF2, (byte)0xD8, (byte)0xD8,
        (byte)0xF1, (byte)0xD7, (byte)0xD7, (byte)0xF0, (byte)0xD4, (byte)0xD4,
        (byte)0xED, (byte)0xD3, (byte)0xD3, (byte)0xEC, (byte)0xD2, (byte)0xD2,
        (byte)0xEB, (byte)0xD1, (byte)0xD1, (byte)0xEA, (byte)0xD0, (byte)0xD0,
        (byte)0xE9, (byte)0xCD, (byte)0xCD, (byte)0xE7, (byte)0xCD, (byte)0xCD,
        (byte)0xE6, (byte)0xCC, (byte)0xCC, (byte)0xFF, (byte)0xCC, (byte)0xCC,
        (byte)0xE6, (byte)0xB3, (byte)0xB3, (byte)0xCC, (byte)0xB3, (byte)0xB2,
        (byte)0xCC, (byte)0xB2, (byte)0xB3, (byte)0xCC, (byte)0xB2, (byte)0xB2,
        (byte)0xCC, (byte)0xB2, (byte)0xB2, (byte)0xCB, (byte)0xB1, (byte)0xB1,
        (byte)0xCB, (byte)0xB0, (byte)0xB0, (byte)0xCA, (byte)0xB0, (byte)0xAF,
        (byte)0xC9, (byte)0xAF, (byte)0xB0, (byte)0xC9, (byte)0xAE, (byte)0xAF,
        (byte)0xC8, (byte)0xAE, (byte)0xAE, (byte)0xC8, (byte)0xAE, (byte)0xAE,
        (byte)0xC7, (byte)0xAD, (byte)0xAD, (byte)0xC6, (byte)0xAC, (byte)0xAC,
        (byte)0xC5, (byte)0xAB, (byte)0xAB, (byte)0xC4, (byte)0xAA, (byte)0xAA,
        (byte)0xC4, (byte)0xA9, (byte)0xA9, (byte)0xC2, (byte)0xA7, (byte)0xA7,
        (byte)0xC1, (byte)0xA6, (byte)0xA6, (byte)0xC0, (byte)0xA5, (byte)0xA6,
        (byte)0xBF, (byte)0xA4, (byte)0xA5, (byte)0xBE, (byte)0xA4, (byte)0xA4,
        (byte)0xBE, (byte)0xA3, (byte)0xA4, (byte)0xBD, (byte)0xA3, (byte)0xA3,
        (byte)0xBD, (byte)0xA2, (byte)0xA2, (byte)0xBC, (byte)0xA0, (byte)0xA0,
        (byte)0xBA, (byte)0x9F, (byte)0x9F, (byte)0xB9, (byte)0x9E, (byte)0x9E,
        (byte)0xB8, (byte)0x9D, (byte)0x9D, (byte)0xB7, (byte)0x9B, (byte)0x9B,
        (byte)0xB4, (byte)0x9A, (byte)0x9A, (byte)0xB3, (byte)0x99, (byte)0x99,
        (byte)0xB3, (byte)0x99, (byte)0x99, (byte)0xB2, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0xC0, (byte)0xC0, (byte)0xC0, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x21, (byte)0xF9, (byte)0x04, (byte)0x01, (byte)0x00,
        (byte)0x00, (byte)0x40, (byte)0x00, (byte)0x2C, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x18, (byte)0x00, (byte)0x18, (byte)0x00,
        (byte)0x40, (byte)0x07, (byte)0xB4, (byte)0x80, (byte)0x40, (byte)0x82,
        (byte)0x83, (byte)0x84, (byte)0x85, (byte)0x86, (byte)0x86, (byte)0x3F,
        (byte)0x00, (byte)0x1A, (byte)0x31, (byte)0x0C, (byte)0x89, (byte)0x3A,
        (byte)0x1A, (byte)0x8F, (byte)0x3F, (byte)0x1C, (byte)0x1C, (byte)0x3F,
        (byte)0x87, (byte)0x82, (byte)0x3F, (byte)0x98, (byte)0x99, (byte)0x00,
        (byte)0x27, (byte)0x01, (byte)0x1E, (byte)0x9D, (byte)0x01, (byte)0x29,
        (byte)0x14, (byte)0x3A, (byte)0x89, (byte)0x88, (byte)0x8A, (byte)0x8F,
        (byte)0xA3, (byte)0x93, (byte)0x95, (byte)0x96, (byte)0x85, (byte)0x89,
        (byte)0x9F, (byte)0x8E, (byte)0x8B, (byte)0x0E, (byte)0x9C, (byte)0x9E,
        (byte)0x98, (byte)0xAB, (byte)0x83, (byte)0x99, (byte)0xB7, (byte)0xB8,
        (byte)0x00, (byte)0x88, (byte)0xB8, (byte)0x99, (byte)0xB5, (byte)0xAC,
        (byte)0xA6, (byte)0x15, (byte)0xBC, (byte)0xBD, (byte)0xAB, (byte)0x89,
        (byte)0x0E, (byte)0xB3, (byte)0x00, (byte)0xA7, (byte)0x15, (byte)0x31,
        (byte)0x01, (byte)0x92, (byte)0xAA, (byte)0xA5, (byte)0xAE, (byte)0x00,
        (byte)0x31, (byte)0x0E, (byte)0x25, (byte)0x9F, (byte)0x9E, (byte)0x29,
        (byte)0xA4, (byte)0xB5, (byte)0x89, (byte)0x9E, (byte)0xD6, (byte)0x14,
        (byte)0x38, (byte)0x90, (byte)0xD8, (byte)0xBE, (byte)0xE1, (byte)0xE2,
        (byte)0xE3, (byte)0x97, (byte)0x00, (byte)0xE6, (byte)0xE7, (byte)0xB4,
        (byte)0xE3, (byte)0xAF, (byte)0x3A, (byte)0x15, (byte)0x2F, (byte)0xCD,
        (byte)0xEA, (byte)0xC8, (byte)0x8B, (byte)0x89, (byte)0xE7, (byte)0xE6,
        (byte)0xE9, (byte)0x96, (byte)0x89, (byte)0xD2, (byte)0x1E, (byte)0x89,
        (byte)0x90, (byte)0x90, (byte)0xC1, (byte)0x94, (byte)0xC4, (byte)0x00,
        (byte)0xA8, (byte)0xE9, (byte)0x33, (byte)0xB5, (byte)0x28, (byte)0x00,
        (byte)0x35, (byte)0x7B, (byte)0xBF, (byte)0xB6, (byte)0xED, (byte)0xCB,
        (byte)0x57, (byte)0x8D, (byte)0x99, (byte)0xAE, (byte)0x84, (byte)0xA0,
        (byte)0x8A, (byte)0x99, (byte)0x68, (byte)0xF8, (byte)0x22, (byte)0xD8,
        (byte)0xC3, (byte)0x42, (byte)0xF4, (byte)0x5A, (byte)0x6D, (byte)0x03,
        (byte)0xD5, (byte)0x4D, (byte)0x03, (byte)0xB8, (byte)0x7B, (byte)0x00,
        (byte)0x3E, (byte)0x9D, (byte)0xA0, (byte)0x10, (byte)0x83, (byte)0xDF,
        (byte)0x47, (byte)0x4B, (byte)0xF4, (byte)0x52, (byte)0x9E, (byte)0x23,
        (byte)0x17, (byte)0x2E, (byte)0x10, (byte)0x00, (byte)0x21, (byte)0xFE,
        (byte)0x4F, (byte)0x43, (byte)0x6F, (byte)0x70, (byte)0x79, (byte)0x72,
        (byte)0x69, (byte)0x67, (byte)0x68, (byte)0x74, (byte)0x20, (byte)0x32,
        (byte)0x30, (byte)0x30, (byte)0x30, (byte)0x20, (byte)0x62, (byte)0x79,
        (byte)0x20, (byte)0x53, (byte)0x75, (byte)0x6E, (byte)0x20, (byte)0x4D,
        (byte)0x69, (byte)0x63, (byte)0x72, (byte)0x6F, (byte)0x73, (byte)0x79,
        (byte)0x73, (byte)0x74, (byte)0x65, (byte)0x6D, (byte)0x73, (byte)0x2C,
        (byte)0x20, (byte)0x49, (byte)0x6E, (byte)0x63, (byte)0x2E, (byte)0x20,
        (byte)0x41, (byte)0x6C, (byte)0x6C, (byte)0x20, (byte)0x52, (byte)0x69,
        (byte)0x67, (byte)0x68, (byte)0x74, (byte)0x73, (byte)0x20, (byte)0x52,
        (byte)0x65, (byte)0x73, (byte)0x65, (byte)0x72, (byte)0x76, (byte)0x65,
        (byte)0x64, (byte)0x2E, (byte)0x0D, (byte)0x0A, (byte)0x4A, (byte)0x4C,
        (byte)0x46, (byte)0x20, (byte)0x47, (byte)0x52, (byte)0x20, (byte)0x56,
        (byte)0x65, (byte)0x72, (byte)0x20, (byte)0x31, (byte)0x2E, (byte)0x30,
        (byte)0x0D, (byte)0x0A, (byte)0x00, (byte)0x3B
    };

    /**
     * The copy icon.
     */
    public static final Icon COPY_ICON = new ImageIcon(
        TOOLKIT.createImage(COPY_ICON_BYTES),
        "Copy"
        );

    /**
     * The bytes for the paste icon.
     */
    public static final byte PASTE_ICON_BYTES[] =
    {
        (byte)0x47, (byte)0x49, (byte)0x46, (byte)0x38, (byte)0x39, (byte)0x61,
        (byte)0x18, (byte)0x00, (byte)0x18, (byte)0x00, (byte)0xC4, (byte)0xFF,
        (byte)0x00, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xE8, (byte)0xD2,
        (byte)0xBC, (byte)0xE8, (byte)0xD1, (byte)0xBC, (byte)0xE7, (byte)0xD1,
        (byte)0xBC, (byte)0xE7, (byte)0xD1, (byte)0xBB, (byte)0xDB, (byte)0xC8,
        (byte)0xB5, (byte)0xDB, (byte)0xC8, (byte)0xB4, (byte)0xDB, (byte)0xC7,
        (byte)0xB4, (byte)0xDA, (byte)0xC7, (byte)0xB4, (byte)0xCE, (byte)0xBD,
        (byte)0xAD, (byte)0xCC, (byte)0xCC, (byte)0xCC, (byte)0xC2, (byte)0xC1,
        (byte)0xC2, (byte)0xC1, (byte)0xC1, (byte)0xC1, (byte)0xC0, (byte)0xC0,
        (byte)0xC0, (byte)0xB5, (byte)0x9F, (byte)0x89, (byte)0xB5, (byte)0x9E,
        (byte)0x89, (byte)0xB4, (byte)0x9E, (byte)0x89, (byte)0xAE, (byte)0x9A,
        (byte)0x86, (byte)0xAE, (byte)0x9A, (byte)0x85, (byte)0xAE, (byte)0x99,
        (byte)0x86, (byte)0xAE, (byte)0x99, (byte)0x85, (byte)0xA0, (byte)0x8F,
        (byte)0x7D, (byte)0xA0, (byte)0x8E, (byte)0x7D, (byte)0x9A, (byte)0x8A,
        (byte)0x7A, (byte)0x99, (byte)0x99, (byte)0x99, (byte)0x63, (byte)0x63,
        (byte)0x63, (byte)0x63, (byte)0x31, (byte)0x31, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0xC0, (byte)0xC0, (byte)0xC0, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x21, (byte)0xF9, (byte)0x04, (byte)0x01, (byte)0x00,
        (byte)0x00, (byte)0x1C, (byte)0x00, (byte)0x2C, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x18, (byte)0x00, (byte)0x18, (byte)0x00,
        (byte)0x40, (byte)0x05, (byte)0xA0, (byte)0x20, (byte)0x27, (byte)0x8E,
        (byte)0x64, (byte)0x69, (byte)0x9E, (byte)0xDC, (byte)0x06, (byte)0x24,
        (byte)0x17, (byte)0x9B, (byte)0x54, (byte)0x47, (byte)0x34, (byte)0x38,
        (byte)0x1A, (byte)0x80, (byte)0x8A, (byte)0xEA, (byte)0x3C, (byte)0x07,
        (byte)0xD1, (byte)0x51, (byte)0xB1, (byte)0xB5, (byte)0x1B, (byte)0x4E,
        (byte)0x93, (byte)0x01, (byte)0x60, (byte)0x16, (byte)0x8A, (byte)0x45,
        (byte)0x46, (byte)0xB3, (byte)0x11, (byte)0xE2, (byte)0x56, (byte)0x30,
        (byte)0x99, (byte)0x63, (byte)0x17, (byte)0x09, (byte)0xDE, (byte)0x54,
        (byte)0xBC, (byte)0xC2, (byte)0xAF, (byte)0xC5, (byte)0xB5, (byte)0x3A,
        (byte)0x39, (byte)0x45, (byte)0x00, (byte)0x20, (byte)0xF3, (byte)0x1D,
        (byte)0xA9, (byte)0x34, (byte)0xE8, (byte)0x74, (byte)0xBA, (byte)0x6A,
        (byte)0xBB, (byte)0xAE, (byte)0x5A, (byte)0x51, (byte)0x1D, (byte)0xAD,
        (byte)0x8D, (byte)0x52, (byte)0xC5, (byte)0xE4, (byte)0x83, (byte)0x88,
        (byte)0xD6, (byte)0x7B, (byte)0xCA, (byte)0x4D, (byte)0x7B, (byte)0x5B,
        (byte)0x40, (byte)0x74, (byte)0x37, (byte)0x62, (byte)0x85, (byte)0x86,
        (byte)0x62, (byte)0x65, (byte)0x23, (byte)0x19, (byte)0x8B, (byte)0x89,
        (byte)0x25, (byte)0x61, (byte)0x1B, (byte)0x48, (byte)0x64, (byte)0x65,
        (byte)0x67, (byte)0x62, (byte)0x48, (byte)0x0B, (byte)0x18, (byte)0x18,
        (byte)0x35, (byte)0x1A, (byte)0x4E, (byte)0x2A, (byte)0x5C, (byte)0x2E,
        (byte)0x71, (byte)0x7C, (byte)0x26, (byte)0x9D, (byte)0x9F, (byte)0x77,
        (byte)0x53, (byte)0xA1, (byte)0x25, (byte)0xA3, (byte)0x71, (byte)0x53,
        (byte)0xA6, (byte)0x83, (byte)0xA2, (byte)0x00, (byte)0xAA, (byte)0x3A,
        (byte)0x3D, (byte)0xA7, (byte)0x24, (byte)0x2A, (byte)0x52, (byte)0x54,
        (byte)0x5A, (byte)0x09, (byte)0xB3, (byte)0x66, (byte)0x00, (byte)0xAB,
        (byte)0x79, (byte)0xB8, (byte)0x2D, (byte)0xBA, (byte)0x4F, (byte)0x0E,
        (byte)0x3C, (byte)0x3E, (byte)0x2E, (byte)0xC0, (byte)0xAD, (byte)0x26,
        (byte)0x6A, (byte)0xCA, (byte)0x68, (byte)0x88, (byte)0x8D, (byte)0x65,
        (byte)0x21, (byte)0x00, (byte)0x21, (byte)0xFE, (byte)0x4F, (byte)0x43,
        (byte)0x6F, (byte)0x70, (byte)0x79, (byte)0x72, (byte)0x69, (byte)0x67,
        (byte)0x68, (byte)0x74, (byte)0x20, (byte)0x32, (byte)0x30, (byte)0x30,
        (byte)0x30, (byte)0x20, (byte)0x62, (byte)0x79, (byte)0x20, (byte)0x53,
        (byte)0x75, (byte)0x6E, (byte)0x20, (byte)0x4D, (byte)0x69, (byte)0x63,
        (byte)0x72, (byte)0x6F, (byte)0x73, (byte)0x79, (byte)0x73, (byte)0x74,
        (byte)0x65, (byte)0x6D, (byte)0x73, (byte)0x2C, (byte)0x20, (byte)0x49,
        (byte)0x6E, (byte)0x63, (byte)0x2E, (byte)0x20, (byte)0x41, (byte)0x6C,
        (byte)0x6C, (byte)0x20, (byte)0x52, (byte)0x69, (byte)0x67, (byte)0x68,
        (byte)0x74, (byte)0x73, (byte)0x20, (byte)0x52, (byte)0x65, (byte)0x73,
        (byte)0x65, (byte)0x72, (byte)0x76, (byte)0x65, (byte)0x64, (byte)0x2E,
        (byte)0x0D, (byte)0x0A, (byte)0x4A, (byte)0x4C, (byte)0x46, (byte)0x20,
        (byte)0x47, (byte)0x52, (byte)0x20, (byte)0x56, (byte)0x65, (byte)0x72,
        (byte)0x20, (byte)0x31, (byte)0x2E, (byte)0x30, (byte)0x0D, (byte)0x0A,
        (byte)0x00, (byte)0x3B
    };

    /**
     * The paste icon.
     */
    public static final Icon PASTE_ICON = new ImageIcon(
        TOOLKIT.createImage(PASTE_ICON_BYTES),
        "Paste"
        );

    /**
     * The bytes for the select all icon.
     */
    public static final byte SELECT_ALL_ICON_BYTES[] =
    {
        (byte)0x47, (byte)0x49, (byte)0x46, (byte)0x38, (byte)0x39, (byte)0x61,
        (byte)0x18, (byte)0x00, (byte)0x18, (byte)0x00, (byte)0xF7, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x0D, (byte)0x0D,
        (byte)0x0D, (byte)0x9D, (byte)0x9D, (byte)0xB7, (byte)0x9F, (byte)0x9F,
        (byte)0xB9, (byte)0xA5, (byte)0xA6, (byte)0xBF, (byte)0xA7, (byte)0xA7,
        (byte)0xC1, (byte)0xAE, (byte)0xAE, (byte)0xC7, (byte)0xAE, (byte)0xAF,
        (byte)0xC8, (byte)0xAF, (byte)0xB0, (byte)0xC9, (byte)0xB0, (byte)0xAF,
        (byte)0xC9, (byte)0xB3, (byte)0xB3, (byte)0xCC, (byte)0xC0, (byte)0xC0,
        (byte)0xC0, (byte)0xCD, (byte)0xCD, (byte)0xE7, (byte)0xCC, (byte)0xCC,
        (byte)0xFF, (byte)0xD4, (byte)0xD4, (byte)0xED, (byte)0xD7, (byte)0xD7,
        (byte)0xF0, (byte)0xDD, (byte)0xDD, (byte)0xF6, (byte)0xE7, (byte)0xE7,
        (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xC0, (byte)0xC0,
        (byte)0xC0, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x21, (byte)0xF9, (byte)0x04, (byte)0x01, (byte)0x00,
        (byte)0x00, (byte)0x13, (byte)0x00, (byte)0x2C, (byte)0x00, (byte)0x00,
        (byte)0x00, (byte)0x00, (byte)0x18, (byte)0x00, (byte)0x18, (byte)0x00,
        (byte)0x00, (byte)0x08, (byte)0xB4, (byte)0x00, (byte)0x27, (byte)0x08,
        (byte)0x1C, (byte)0x48, (byte)0xB0, (byte)0xA0, (byte)0x41, (byte)0x82,
        (byte)0x01, (byte)0x02, (byte)0x08, (byte)0x4C, (byte)0xA8, (byte)0x70,
        (byte)0x02, (byte)0xC3, (byte)0x85, (byte)0x09, (byte)0x21, (byte)0x0E,
        (byte)0x6C, (byte)0x78, (byte)0xB0, (byte)0xE2, (byte)0x42, (byte)0x8B,
        (byte)0x18, (byte)0x33, (byte)0x0A, (byte)0x04, (byte)0xC0, (byte)0xB1,
        (byte)0x63, (byte)0x47, (byte)0x8C, (byte)0x14, (byte)0x27, (byte)0x00,
        (byte)0x90, (byte)0x40, (byte)0xB2, (byte)0x24, (byte)0x47, (byte)0x83,
        (byte)0x14, (byte)0x43, (byte)0x8E, (byte)0x14, (byte)0xC0, (byte)0x80,
        (byte)0x25, (byte)0x80, (byte)0x06, (byte)0x0D, (byte)0x00, (byte)0x14,
        (byte)0x4C, (byte)0x49, (byte)0x70, (byte)0x64, (byte)0xCB, (byte)0x96,
        (byte)0x0E, (byte)0x5E, (byte)0xCA, (byte)0x44, (byte)0x58, (byte)0x71,
        (byte)0x25, (byte)0x4E, (byte)0x02, (byte)0x11, (byte)0x74, (byte)0x6A,
        (byte)0x14, (byte)0x29, (byte)0xE1, (byte)0x26, (byte)0xD0, (byte)0x04,
        (byte)0x41, (byte)0x77, (byte)0xA2, (byte)0xAC, (byte)0x29, (byte)0x81,
        (byte)0x25, (byte)0x01, (byte)0x08, (byte)0x07, (byte)0x22, (byte)0x28,
        (byte)0x48, (byte)0x3A, (byte)0xB1, (byte)0xEA, (byte)0x40, (byte)0x9B,
        (byte)0x4F, (byte)0xA7, (byte)0x6A, (byte)0x0D, (byte)0x6A, (byte)0xD5,
        (byte)0x21, (byte)0xD3, (byte)0xA7, (byte)0x48, (byte)0xB7, (byte)0x1A,
        (byte)0x50, (byte)0x1A, (byte)0x92, (byte)0x29, (byte)0x04, (byte)0x04,
        (byte)0x52, (byte)0xD3, (byte)0x16, (byte)0xC8, (byte)0xA9, (byte)0x71,
        (byte)0x64, (byte)0x54, (byte)0xB1, (byte)0x0F, (byte)0x04, (byte)0x28,
        (byte)0xE5, (byte)0x79, (byte)0x55, (byte)0x42, (byte)0xDA, (byte)0xA9,
        (byte)0x06, (byte)0x1E, (byte)0x0C, (byte)0x60, (byte)0x40, (byte)0xB6,
        (byte)0x2B, (byte)0x51, (byte)0xB8, (byte)0x7B, (byte)0xE5, (byte)0x76,
        (byte)0x55, (byte)0x69, (byte)0x77, (byte)0xEA, (byte)0x81, (byte)0x07,
        (byte)0x04, (byte)0x6E, (byte)0xF6, (byte)0x3D, (byte)0xE8, (byte)0xB1,
        (byte)0xF1, (byte)0xC7, (byte)0xA1, (byte)0x90, (byte)0xFD, (byte)0x46,
        (byte)0x4E, (byte)0xD9, (byte)0xF0, (byte)0xA1, (byte)0xC3, (byte)0x88,
        (byte)0x97, (byte)0x2B, (byte)0x97, (byte)0x8D, (byte)0xCC, (byte)0x19,
        (byte)0x72, (byte)0x40, (byte)0x00, (byte)0x00, (byte)0x3B
    };

    /**
     * The paste icon.
     */
    public static final Icon SELECT_ALL_ICON = new ImageIcon(
        TOOLKIT.createImage(SELECT_ALL_ICON_BYTES),
        "Select All"
        );

    /**
     * The keystroke generally associated with the cut action.
     */
    public static final KeyStroke CUT_KEYSTROKE =
        KeyStroke.getKeyStroke(KeyEvent.VK_X, TOOLKIT.getMenuShortcutKeyMask());

    /**
     * The keystroke generally associated with the copy action.
     */
    public static final KeyStroke COPY_KEYSTROKE =
        KeyStroke.getKeyStroke(KeyEvent.VK_C, TOOLKIT.getMenuShortcutKeyMask());

    /**
     * The keystroke generally associated with the paste action.
     */
    public static final KeyStroke PASTE_KEYSTROKE =
        KeyStroke.getKeyStroke(KeyEvent.VK_V, TOOLKIT.getMenuShortcutKeyMask());

    /**
     * The keystroke generally associated with the select all action.
     */
    public static final KeyStroke SELECT_ALL_KEYSTROKE =
        KeyStroke.getKeyStroke(KeyEvent.VK_A, TOOLKIT.getMenuShortcutKeyMask());

    /**
     * An implementation of Action to use for the cut operation.  The ClipboardTargetManager will
     * manage the enabled status of this Action.
     */
    public static final AbstractAction2 CUT_ACTION = new CutAction();

    /**
     * An implementation of Action to use for the copy operation.  The ClipboardTargetManager will
     * manage the enabled status of this Action.
     */
    public static final AbstractAction2 COPY_ACTION = new CopyAction();

    /**
     * An implementation of Action to use for the paste operation.  The ClipboardTargetManager will
     * manage the enabled status of this Action.
     */
    public static final AbstractAction2 PASTE_ACTION = new PasteAction();

    /**
     * An implementation of Action to use for the selectAll operation.  The ClipboardTargetManager
     * will manage the enabled status of this Action.
     */
    public static final AbstractAction2 SELECT_ALL_ACTION = new SelectAllAction();

    private static final ClipboardTargetManager INSTANCE = new ClipboardTargetManager();

    private ClipboardTarget lastFocusedTarget;

    /**
     * Constructs a new ClipboardTargetManager.
     */
    private ClipboardTargetManager()
    {
        lastFocusedTarget = null;
    }

    /**
     * Installs the InputMap and ActionMap mappings necessary to get the JComponent to behave
     * correctly when the keyboard trigger for a clipboard action is fired.
     *
     * @param component The component for which we should install the mappings.  A
     * NullPointerException will be thrown if this is null.
     */
    public static void installMappings(JComponent component)
    {
        InputMap inputMap = component.getInputMap();
        ActionMap actionMap = component.getActionMap();
        
        Object cutKey = inputMap.get(ClipboardTargetManager.CUT_KEYSTROKE);
        if (cutKey == null)
            cutKey = "cut";
        inputMap.put(ClipboardTargetManager.CUT_KEYSTROKE, cutKey);
        actionMap.put(cutKey, ClipboardTargetManager.CUT_ACTION);

        Object copyKey = inputMap.get(ClipboardTargetManager.COPY_KEYSTROKE);
        if (copyKey == null || copyKey.equals(cutKey))
            copyKey = "copy";
        inputMap.put(ClipboardTargetManager.COPY_KEYSTROKE, copyKey);
        actionMap.put(copyKey, ClipboardTargetManager.COPY_ACTION);

        Object pasteKey = inputMap.get(ClipboardTargetManager.PASTE_KEYSTROKE);
        if (pasteKey == null || pasteKey.equals(cutKey) || pasteKey.equals(copyKey))
            pasteKey = "paste";
        inputMap.put(ClipboardTargetManager.PASTE_KEYSTROKE, pasteKey);
        actionMap.put(pasteKey, ClipboardTargetManager.PASTE_ACTION);

        Object selectAllKey = inputMap.get(ClipboardTargetManager.SELECT_ALL_KEYSTROKE);
        if (selectAllKey == null || selectAllKey.equals(cutKey) || selectAllKey.equals(copyKey) ||
            selectAllKey.equals(pasteKey))
        {
            selectAllKey = "selectAll";
        }
        inputMap.put(ClipboardTargetManager.SELECT_ALL_KEYSTROKE, selectAllKey);
        actionMap.put(selectAllKey, ClipboardTargetManager.SELECT_ALL_ACTION);
    }

    /**
     * Creates a basic JPopupMenu which contains MenuItems for all the clipboard actions.
     *
     * @return A basic JPopupMenu which contains MenuItems for all the clipboard actions.
     */
    public static JPopupMenu createDefaultPopupMenu()
    {
        JPopupMenu popupMenu = new JPopupMenu();
        popupMenu.add(CUT_ACTION.createMenuItem());
        popupMenu.add(COPY_ACTION.createMenuItem());
        popupMenu.add(PASTE_ACTION.createMenuItem());
        popupMenu.addSeparator();
        popupMenu.add(SELECT_ALL_ACTION.createMenuItem());

        return popupMenu;
    }

    /**
     * The function by which a ClipboardTarget registers itself with the Manager when it gains the
     * focus.
     *
     * @param target The ClipboardTarget which just gained the focus.
     */
    public static void targetGainedFocus(ClipboardTarget target)
    {
        if (target == null)
            throw new NullPointerException();

        synchronized (INSTANCE)
        {
            INSTANCE.lastFocusedTarget = target;

            CUT_ACTION.setEnabled(target.canCut());
            COPY_ACTION.setEnabled(target.canCopy());
            PASTE_ACTION.setEnabled(target.canPaste());
            SELECT_ALL_ACTION.setEnabled(target.canSelectAll());
        }
    }

    /**
     * The function by which a ClipboardTarget unregisters itself with the Manager when it loses the
     * focus.
     *
     * @param target The ClipboardTarget which just lost the focus.
     */
    public static void targetLostFocus(ClipboardTarget target)
    {
        if (target == null)
            throw new NullPointerException();

        // We could do something with this in the future
    }

    /**
     * The function by which a ClipboardTarget registers itself with the Manager when its canX()
     * method changes.
     *
     * @param target The ClipboardTarget which just lost the focus.
     */
    public static void targetAbilityChanged(ClipboardTarget target)
    {
        if (target == null)
            throw new NullPointerException();

        synchronized (INSTANCE)
        {
            if (INSTANCE.lastFocusedTarget == target)
            {
                CUT_ACTION.setEnabled(target.canCut());
                COPY_ACTION.setEnabled(target.canCopy());
                PASTE_ACTION.setEnabled(target.canPaste());
                SELECT_ALL_ACTION.setEnabled(target.canSelectAll());
            }
        }
    }

    /**
     * Returns the ClipboardTarget which most recently owned the focus.  If no such ClipboardTarget
     * exists, null is returned.  If that target (or one of its parents) is not currently showing,
     * then null will be returned.
     *
     * @return The ClipboardTarget which most recently owned the focus.
     */
    public static ClipboardTarget getRecentTarget()
    {
        synchronized (INSTANCE)
        {
            ClipboardTarget target = INSTANCE.lastFocusedTarget;
            if (target != null)
            {
                // Check the visibility of the target
                if (target instanceof Component)
                {
                    Component component = (Component)target;

                    while (component != null)
                    {
                        if (!component.isVisible())
                            return null;
                        component = component.getParent();
                    }
                }

                return target;
            }
            else {
                return null;
            }
        }
    }

    /**
     * Executes the cut() method of the ClipboardTarget which most recently owned the focus.
     * If no such ClipboardTarget exists or if that target is not showing, nothing will happen.
     */
    public static void cut()
    {
        ClipboardTarget target = getRecentTarget();

        if (target != null)
            target.cut();
    }

    /**
     * Executes the copy() method of the ClipboardTarget which most recently owned the focus.
     * If no such ClipboardTarget exists or if that target is not showing, nothing will happen.
     */
    public static void copy()
    {
        ClipboardTarget target = getRecentTarget();

        if (target != null)
            target.copy();
    }

    /**
     * Executes the paste() method of the ClipboardTarget which most recently owned the focus.
     * If no such ClipboardTarget exists or if that target is not showing, nothing will happen.
     */
    public static void paste()
    {
        ClipboardTarget target = getRecentTarget();

        if (target != null)
            target.paste();
    }

    /**
     * Executes the selectAll() method of the ClipboardTarget which most recently owned the focus.
     * If no such ClipboardTarget exists or if that target is not showing, nothing will happen.
     */
    public static void selectAll()
    {
        ClipboardTarget target = getRecentTarget();

        if (target != null)
            target.selectAll();
    }

    /**
     * CutAction is an action which calls the cut() method of the ClipboardTargetManager.  The
     * ClipboardTargetManager also manages this action's enabled status.
     *
     * @author btsang
     * @version 7.1
     */
    protected static class CutAction extends AbstractAction2
    {
        /**
         * Creates a new CutAction.
         */
        protected CutAction()
        {
            super("Cut", CUT_ICON);

            setAccelerator(CUT_KEYSTROKE);

            setMnemonic(KeyEvent.VK_C);
        }

        /**
         * Implements ActionListener to perform the cut operation.
         */
        public void actionPerformed(ActionEvent e)
        {
            ClipboardTargetManager.cut();
        }
    }

    /**
     * CopyAction is an action which calls the copy() method of the ClipboardTargetManager.  The
     * ClipboardTargetManager also manages this action's enabled status.
     *
     * @author btsang
     * @version 7.1
     */
    protected static class CopyAction extends AbstractAction2
    {
        /**
         * Creates a new CopyAction.
         */
        protected CopyAction()
        {
            super("Copy", COPY_ICON);

            setAccelerator(COPY_KEYSTROKE);

            setMnemonic(KeyEvent.VK_C);
        }

        /**
         * Implements ActionListener to perform the cut operation.
         */
        public void actionPerformed(ActionEvent e)
        {
            ClipboardTargetManager.copy();
        }
    }

    /**
     * PasteAction is an action which calls the paste() method of the ClipboardTargetManager.  The
     * ClipboardTargetManager also manages this action's enabled status.
     *
     * @author btsang
     * @version 7.1
     */
    protected static class PasteAction extends AbstractAction2
    {
        /**
         * Creates a new PasteAction.
         */
        protected PasteAction()
        {
            super("Paste", PASTE_ICON);

            setAccelerator(PASTE_KEYSTROKE);

            setMnemonic(KeyEvent.VK_P);
        }

        /**
         * Implements ActionListener to perform the cut operation.
         */
        public void actionPerformed(ActionEvent e)
        {
            ClipboardTargetManager.paste();
        }
    }

    /**
     * SelectAllAction is an action which calls the selectAll() method of the
     * ClipboardTargetManager.  The ClipboardTargetManager also manages this action's enabled
     * status.
     *
     * @author btsang
     * @version 7.1
     */
    protected static class SelectAllAction extends AbstractAction2
    {
        /**
         * Creates a new SelectAllAction.
         */
        protected SelectAllAction()
        {
            super("Select All", SELECT_ALL_ICON);

            setAccelerator(SELECT_ALL_KEYSTROKE);

            setMnemonic(KeyEvent.VK_A);
        }

        /**
         * Implements ActionListener to perform the cut operation.
         */
        public void actionPerformed(ActionEvent e)
        {
            ClipboardTargetManager.selectAll();
        }
    }
}
